const got = require('got');
const vorpal = require('vorpal')();
const querystring = require("querystring");
const shortuuid = require('short-uuid');
const fs = require('fs');
const Pusher = require('pusher-js');
const path = require('path');

APP_KEY = "2c20398a6efc08ba4bce"
APP_CLUSTER = "us3"

const socket = new Pusher(APP_KEY, {
  cluster: APP_CLUSTER,
  forceTLS: true
});

global._accessToken = "test"
global._refreshToken = ""
global._userId = ""
global._pw = ""
global._remember = "no"
global._jobid = ""
global._channel = null;
global._vorpal = null;

global._API = 'https://430y0f3m75.execute-api.us-west-2.amazonaws.com/api'
// global._API = "http://localhost:8000"


vorpal
  .command('signup', 'create new account')
  .alias('register')
  .alias('sign up')
  .alias('new account')
  .alias('new user')
  .action(function (params, cb) {
    this.prompt([
      {
        type: 'input',
        name: 'username',
        message: 'username:  '
      },
      {
        type: 'password',
        name: 'password',
        message: 'password:  '
      },
      {
        type: 'input',
        name: 'email',
        message: 'email:  '
      }]).then(async function (result) {

        try {

          const myresult = await got.post(_API + '/register', {
            'body': JSON.stringify({
              'user': result.username,
              'pass': result.password,
              'email': result.email
            }),
            'headers': { 'Content-Type': 'application/json' },
            'responseType': 'json'
          });


          
          if (myresult['body'].startsWith('Fail')) {
            console.log(myresult['body'])
          } else {
            console.log('Successfully registered. Please type login to continue.');
          }

          cb();
        } catch (e) {
          console.log('Please make sure your username and password is at least 6 characters long.');
          cb();
        }
      });
  });

vorpal
  .command('signin', 'signin to your account')
  .alias('login')
  .alias('log in')
  .alias('sign in')
  .action(function (params, cb) {
    this.prompt([
      {
        type: 'input',
        name: 'username',
        message: 'username:  '
      },
      {
        type: 'password',
        name: 'password',
        message: 'password:  '
      }
    ]).then(async function (result) {
      try {
        const myresult = await got.post(_API + '/login', {
          'body': JSON.stringify({
            'user': result.username,
            'pass': result.password
          }),
          'headers': { 'Content-Type': 'application/json' },
          'responseType': 'json'
        });

        
        if (myresult['body'].startsWith('Fail')) {
          console.log(myresult['body'])
          _vorpal.show();
          cb();
          return;
        }

        _accessToken = JSON.parse(myresult["body"])["access_token"]
        _refreshToken = JSON.parse(myresult["body"])["refresh_token"]
        _userId = JSON.parse(myresult["body"])["username"]
        console.log("SUCCESSFULLY LOGGED IN as " + result.username + ". Type help to see list of available commands.")
        _vorpal.show();
        cb();
      } catch (e) {
        console.log('Failed to login. Please make sure your credentials are correct.');
        cb();
      }
    });
  });

vorpal
  .command('signout', 'sign out of your account')
  .alias('sign out')
  .alias('logout')
  .alias('log out')
  .action(async function (args, callback) {
    this.log('signing out...' + _accessToken);
    try {
      result = await got(_API + '/signout/' + _accessToken)
      this.log(result.body)
      _vorpal.show();
      callback();
    } catch (e) {
      console.log('Please make sure your input is correct');
      callback();
    }
  })

vorpal
  .command('run <wsl...>', 'executes WSL or .wsl file in REPL mode')
  .parse(function (command, args) {
    command = Buffer.from(command.substring(4)).toString('base64')
    return 'run ' + command;
  })
  .action(async function (args, cb) {

    try {
      if (_channel == null) {
        _channel = socket.subscribe(_userId);
        _channel.bind('repl', function (data) {
          console.log(data.message);
          _vorpal.show();
        });
      }

      fwsl = args.wsl
      wsl = args.wsl;
      wsl = wsl.join('')
      msg = {}
      msg["default"] = {}
      message = msg["default"]
      message["event_type"] = _userId + '__job__repl'

      if (".wsl".includes(fwsl)) {
        data = ""
        result = await fs.readFile(data, 'utf8', function (err, contents) {
          console.log(contents);
          return contents
        });
        data = result
        //when reading .wsl file, we replace \n with |
        if (data.includes(">>") == false) {
          data = data.replace(/\r?\n|\r/g, ">>")
          console.log(data)
        }
        message["wsl"] = data
        
        _vorpal.show();

        //we must reset
        //result = await got(_API + '/' + _accessToken + '/' + _userId + '/repl/' + msg)
        const result = await got.post(_API + '/' + _accessToken + '/' + _userId + '/repl', {
          'body': JSON.stringify(msg),
          'headers': { 'Content-Type': 'application/json' },
          'responseType': 'json'
        });
        this.log(result.body)
        _vorpal.show();
        
        cb();
      } else {
        message["wsl"] = wsl
        _vorpal.show();
        //result = await got(_API + '/' + _accessToken + '/' + _userId + '/repl/' + msg)
        const result = await got.post(_API + '/' + _accessToken + '/' + _userId + '/repl', {
          'body': JSON.stringify(msg),
          'headers': { 'Content-Type': 'application/json' },
          'responseType': 'json'
        });

        this.log(result.body)
        _vorpal.show();
        cb();
      }
    } catch (e) {
      // console.log('err msg', e);
      console.log('Please make sure you are signed in and that provided input is correctly formatted.');
      //console.log(e);
      cb();
    }
  })


vorpal
  .command('halt', 'halts REPL')
  .alias('stop')
  .action(async function (args, cb) {
    try {
      this.log('halting execution...');
      result = await got(_API + '/' + _accessToken + '/' + _userId + '/halt')
      this.log(result.body)
      _vorpal.show();
      cb();
    } catch (e) {
      console.log('Please make sure your input is correct');
      cb();
    }
  })

vorpal
  .command('job export <jobid>', 'export data, logs, html cache from job')
  .alias('export <jobid>')
  .action(async function (args, cb) {
    this.log('exporting data...this may take several seconds depending on load');
    var to = ""
    if (args.to !== undefined) {
      to = args.to
    }
    var output = args.output
    var format = args.format

    this.prompt([
      {
        type: 'list',
        name: 'output',
        message: 'What kind of data do you want to export?',
        choices: ['data', 'log', 'error', 'html', 'screenshots'],
        default: 'data'
      },
      {
        type: 'list',
        name: 'format',
        message: 'Choose file format: ',
        choices: ['json', 'csv', 'zip'],
        default: 'json'
      },
      {
        type: 'input',
        name: 'destination',
        message: 'Provide email or url to export to. Leave blank to get download link.',
        default: 'dl'
      }
    ]).then(async function (result) {
      var output = result.output
      var format = result.format
      var to = result.destination

      if (output != "data" && format != "json") {
        console.log("only data output is available in multiple formats. only json is supported.")
        _vorpal.show();
        cb();
      } else if (output == "screenshot" || output == "html") {
        if (format != "zip") {
          console.log("screenshots and html cache is only available in zip format.")
          _vorpal.show();
          cb();
        }
      } else {
        _vorpal.show();
        console.log('exporting may take several seconds...')
        _vorpal.show();
        try {
          myresult = await got(_API + '/' + _accessToken + '/' + _userId + '/export/' + args.jobid +
            '/' + output + '/' + format + '/' + to)
        } catch (e) {
          console.log('Please make sure your input is correct');
          callback();
        }
        _vorpal.show();
        console.log("Export available: " + myresult.body)
        _vorpal.show();
        cb();
      }
    });
  })

vorpal
  .command('job delete <jobid>', 'deletes this job, cannot be undone!')
  .alias('delete')
  .alias('del')
  .action(async function (args, callback) {
    try {
      this.log('scheduling job...');
      result = await got(_API + '/' + _accessToken + '/' + _userId + '/job/' + args.jobid + '/action/delete')
      this.log(result.body)
      _vorpal.show();
      callback();
    } catch (e) {
      console.log('Please make sure your input is correct');
      callback();
    }
  })

vorpal
  .command('job stop <jobid>', 'pauses this job. to resume: job start <jobid>')
  .alias('stop <jobid>')
  .action(async function (args, callback) {
    try {
      this.log('stopping job...');
      result = await got(_API + '/' + _accessToken + '/' + _userId + '/job/' + args.jobid + '/action/stop')
      this.log(result.body)
      callback();
    } catch (e) {
      console.log('Please make sure your input is correct');
      callback();
    }
  })

vorpal
  .command('job start <jobid>', 'starts job')
  .action(async function (args, callback) {
    this.log('starting job');
    result = await got(_API + '/' + _accessToken + '/' + _userId + '/job/' + args.jobid + '/action/start')
    try {
      this.log(result.body)
      _vorpal.show();
      callback();
    } catch (e) {
      console.log('Please make sure your input is correct');
      callback();
    }
  })

vorpal
  .command('job new <wsl...>', 'creates a new job with WSL or from a .wsl file')
  .alias("new <wsl...>")
  .alias("create <wsl...>")
  .action(async function (args, callback) {
    try {
      this.log('creating a new job...');
      wsl = args.wsl;
      if (".wsl".includes(wsl)) {
        data = ""
        result = await fs.readFile(data, 'utf8', function (err, contents) {
          console.log(contents);
          return contents
        });
        data = result
        if (data.includes("|") == false) {
          data = data.replace(/\r?\n|\r/g, "|")
          console.log(data)
        }
        message["wsl"] = data
        //await got(_API + '/' + _accessToken + '/' + _userId + '/job/new/' + data)
        const result = await got.post(_API + '/' + _accessToken + '/' + _userId + '/job/new', {
          'body': JSON.stringify(msg),
          'headers': { 'Content-Type': 'application/json' },
          'responseType': 'json'
        });
        _vorpal.show();
        callback();
      } else {
        message["wsl"] = wsl
        //result = await got(_API + '/' + _accessToken + '/' + _userId + '/job/new/' + wsl)
        const result = await got.post(_API + '/' + _accessToken + '/' + _userId + '/job/new', {
          'body': JSON.stringify(msg),
          'headers': { 'Content-Type': 'application/json' },
          'responseType': 'json'
        });
        this.log(result.body)
        _vorpal.show();
        callback();
      }
    } catch (e) {
      console.log('Please make sure your input is correct');
      callback();
    }
  })

vorpal
  .command('myjobs', 'get status update of all jobs in your account')
  .alias("jobs")
  .alias("my jobs")
  .alias("my job")
  .action(async function (args, callback) {
    try {
      this.log('fetching status of jobs...');
      result = await got(_API + '/' + _accessToken + '/' + _userId + '/jobs')
      this.log(result.body)
      _vorpal.show();
      callback();
    } catch (e) {
      console.log('Please make sure your input is correct');
      callback();
    }
  })

vorpal
  .command('myaccount', 'get details about your account')
  .alias('my account')
  .alias('acct')
  .alias('myacct')
  .action(async function (args, callback) {
    try {
      this.log('fetching my account details');
      result = await got(_API + '/' + _accessToken + '/' + _userId + '/user')
      this.log(result.body)
      _vorpal.show();
      callback();
    } catch (e) {
      console.log('Please make sure your input is correct');
      callback();
    }
  })

vorpal
  .command('mynewpw', 'change your password')
  .alias('newpw')
  .alias('changepassword')
  .alias('changepw')
  .action(async function (args, callback) {
    this.log('changing password...');
    try {
      this.prompt([
        {
          type: 'input',
          name: 'username',
          message: 'Username: '
        },
        {
          type: 'input',
          name: 'password',
          message: 'Existing password: '
        },
        {
          type: 'input',
          name: 'newpassword',
          message: 'New password: '
        }
      ]).then(async function (result) {
        result = await got(_API + '/changepw/' + result.username + '/' + result.password + '/' + result.NewPassword);
        _userId = result.body
        console.log('Successfully changed password.')
        _vorpal.show();
        callback();
      });
    } catch (e) {
      console.log('Please make sure your input is correct');
      callback();
    }
  })

vorpal
  .command('upgrade', 'upgrade to a paid plan')
  .alias('subscribe')
  .alias('buy')
  .alias('checkout')
  .action(async function (args, callback) {
    this.log('fetching your checkout link...');
    try {
      this.prompt([
        {
          type: 'list',
          name: 'plan',
          message: 'Please select a plan to upgrade to:  ',
          choices: ['Micro - $49.99/month', 'Small - $99.99/month', 'Medium - $199.99/month', 'Large - $499/month'],
          default: 'Small - $99.99/month',
        }
      ]).then(async function (result, error) {
        var plan = result.plan.split(' - ')[0];
        console.log(_API + '/' + _accessToken + '/' + _userId + '/upgrade/' + plan)
        result = await got(_API + '/' + _accessToken + '/' + _userId + '/upgrade/' + plan)
        this.log('rr', result.body)
        _vorpal.show();
        callback();
      });
    } catch (e) {
      console.log('Please make sure your input is correct');
      callback();
    }
  })

vorpal
  .command('clear', 'clears screen')
  .alias('cls')
  .alias('clean')
  .action(function (args, cbk) {
    let blank = '';
    for (var i = 0; i < process.stdout.rows; ++i) {
      blank += '\n';
    }
    vorpal.ui.redraw(blank);
    vorpal.ui.redraw('');
    cbk();
  });

_vorpal = vorpal
  .delimiter('wsl$')
  .show();

